@@ -788,7 +788,6 @@ bool Item_nodeset_func_attributebyname::val_native(THD *thd, Native *nodeset)
 bool Item_nodeset_func_predicate::val_native(THD *thd, Native *str)
 {
   Item_nodeset_func *nodeset_func= (Item_nodeset_func*) args[0];
-  Item_func *comp_func= (Item_func*)args[1];
   uint pos= 0, size;
   prepare(thd, str);
   size= (uint)(fltend - fltbeg);
@@ -797,7 +796,7 @@ bool Item_nodeset_func_predicate::val_native(THD *thd, Native *str)
     nodeset_func->context_cache.length(0);
     MY_XPATH_FLT(flt->num, flt->pos, size).
       append_to(&nodeset_func->context_cache);
-    if (comp_func->val_int())
+    if (args[1]->val_int())
       MY_XPATH_FLT(flt->num, pos++).append_to(str);
   }
   return false;
@@ -970,11 +969,16 @@ static Item *create_comparator(MY_XPATH *xpath,
            b->fixed_type_handler() == &type_handler_xpath_nodeset)
   {
     uint len= (uint)(xpath->query.end - context->beg);
-    set_if_smaller(len, 32);
-    my_printf_error(ER_UNKNOWN_ERROR,
-                    "XPATH error: "
-                    "comparison of two nodesets is not supported: '%.*s'",
-                    MYF(0), len, context->beg);
+    if (len <= 32)
+      my_printf_error(ER_UNKNOWN_ERROR,
+                      "XPATH error: "
+                      "comparison of two nodesets is not supported: '%.*s'",
+                      MYF(0), len, context->beg);
+    else
+      my_printf_error(ER_UNKNOWN_ERROR,
+                      "XPATH error: "
+                      "comparison of two nodesets is not supported: '%.32T'",
+                      MYF(0), context->beg);
 
     return 0; // TODO: Comparison of two nodesets
   }
@@ -2627,9 +2631,12 @@ my_xpath_parse_VariableReference(MY_XPATH *xpath)
       xpath->item= NULL;
       DBUG_ASSERT(xpath->query.end > dollar_pos);
       uint len= (uint)(xpath->query.end - dollar_pos);
-      set_if_smaller(len, 32);
-      my_printf_error(ER_UNKNOWN_ERROR, "Unknown XPATH variable at: '%.*s'", 
-                      MYF(0), len, dollar_pos);
+      if (len <= 32)
+        my_printf_error(ER_UNKNOWN_ERROR, "Unknown XPATH variable at: '%.*s'",
+                        MYF(0), len, dollar_pos);
+      else
+        my_printf_error(ER_UNKNOWN_ERROR, "Unknown XPATH variable at: '%.32T'",
+                        MYF(0), dollar_pos);
     }
   }
   return xpath->item ? 1 : 0;
@@ -2760,9 +2767,13 @@ bool Item_xml_str_func::fix_fields(THD *thd, Item **ref)
   if (!rc)
   {
     uint clen= (uint)(xpath.query.end - xpath.lasttok.beg);
-    set_if_smaller(clen, 32);
-    my_printf_error(ER_UNKNOWN_ERROR, "XPATH syntax error: '%.*s'",
-                    MYF(0), clen, xpath.lasttok.beg);
+    if (clen <= 32)
+      my_printf_error(ER_UNKNOWN_ERROR, "XPATH syntax error: '%.*s'",
+                      MYF(0), clen, xpath.lasttok.beg);
+    else
+      my_printf_error(ER_UNKNOWN_ERROR, "XPATH syntax error: '%.32T'",
+                      MYF(0), xpath.lasttok.beg);
+
     return true;
   }
 
